/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.bc.mail.service.provide;

import java.io.IOException;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.messaging.MessagingException;
import javax.mail.Session;
import javax.mail.internet.MimeMessage;

import org.ow2.petals.bc.mail.MailSessionManager;
import org.ow2.petals.bc.mail.MailUtil;
import org.ow2.petals.bc.mail.MimeMessageManager;
import org.ow2.petals.bc.mail.MissingElementException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.w3c.dom.Node;

/**
 * @author Adrien Louis - EBM WebSourcing
 * @author Mathieu CARROLLE - EBM WebSourcing
 */
public class NativeSendService extends AbstractSendService {

    public NativeSendService(MailSessionManager mailSessionManager,
            MimeMessageManager mimeMessageManager, Logger logger) {
        super(mailSessionManager, mimeMessageManager, logger);
    }

    /**
     * Sends mail by using component service
     * 
     * @param exchange
     * @throws JBIException
     * @throws MessagingException
     * @throws IOException
     */
    public void sendMail(final Exchange exchange) throws MessagingException,
            MissingElementException {
        this.logger.info("JBI Exchange received - sendMail service requested");
        ProvideDescriptor provideDescriptor;
        final Node node = exchange.getInMessageContentAsDocument(false).getFirstChild();
        if (MailUtil.checkWsaProperties(exchange)) {
            provideDescriptor = MailUtil.buildFromProperties(exchange, node);
        } else {
            provideDescriptor = MailUtil.buildFromPayload(node);
        }

        // Build the mail session
        final Session session = this.mailSessionManager
                .createSessionPropertiesFromDescriptor(provideDescriptor);
        final MimeMessage mimeMessage = this.mimeMessageManager.mapNormalizedMessageToMimeMessage(
                session, provideDescriptor, exchange, node);
        try {
            this.mailSessionManager.sendMail(mimeMessage, provideDescriptor, session);
        } catch (javax.mail.MessagingException e) {
            throw new MessagingException(e);
        }
        this.logger.info("email sent to " + provideDescriptor.getToAddress());
    }
}
